@@ -1,10 +1,14 @@
 #!perl -w
 
+BEGIN {
+    warn "note: the warning '... will not stay shared' is OK here!!!\n";
+}
+
 # this script will suffer from a closure problem under registry
 # should see it under ::Registry
 # should not see it under ::PerlRun
 
-print "Content-type: text/plain\r\n\r\n";
+print "Content-type: text/plain\n\n";
 
 # this is a closure (when compiled inside handler()):
 my $counter = 0;
