@@ -419,6 +419,7 @@ EOF
 
 my %trace = (
 #    'a' => 'all',
+    'c' => 'configuration for directive handlers',
     'd' => 'directive processing',
     's' => 'perl sections',
     'h' => 'handlers',
@@ -568,7 +569,8 @@ my %sources = (
 
 my @c_src_names = qw(interp tipool log config cmd options callback handler
                      gtop util io filter bucket mgv pcw global env cgi
-                     perl perl_global perl_pp sys);
+                     perl perl_global perl_pp sys module svptr_table
+                     const constants);
 my @g_c_names = map { "modperl_$_" } qw(hooks directives flags xsinit);
 my @c_names   = ('mod_perl', (map "modperl_$_", @c_src_names));
 sub c_files { [map { "$_.c" } @c_names, @g_c_names] }
@@ -718,7 +720,9 @@ my %shortcuts = (
      REDIRECT => 'HTTP_MOVED_TEMPORARILY',
 );
 
-my %ifdef = map { $_, 1 } qw(APLOG_TOCLIENT);
+#backwards compat with older httpd/apr
+#XXX: remove once we require newer httpd/apr
+my %ifdef = map { $_, 1 } qw(APLOG_TOCLIENT APR_LIMIT_NOFILE);
 
 sub constants_ifdef {
     my $name = shift;
