@@ -32,6 +32,15 @@ modperl_interp_t *modperl_interp_get(server_rec *s);
 
 apr_status_t modperl_interp_unselect(void *data);
 
+modperl_interp_t *modperl_interp_pool_get(apr_pool_t *p);
+
+void modperl_interp_pool_set(apr_pool_t *p,
+                             modperl_interp_t *interp,
+                             int cleanup);
+
+modperl_interp_t *modperl_interp_pool_select(apr_pool_t *p,
+                                             server_rec *s);
+
 modperl_interp_t *modperl_interp_select(request_rec *r, conn_rec *c,
                                         server_rec *s);
 
