@@ -5,8 +5,11 @@ use warnings FATAL => 'all';
 
 use Apache::Test;
 use Apache::TestConfig ();
+
 use Apache::Directive ();
 
+use Apache::Const -compile => 'OK';
+
 sub handler {
     my $r = shift;
 
@@ -19,7 +22,6 @@ sub handler {
 
     ok $vars;
 
-
     my $tree = Apache::Directive->conftree;
 
     ok $tree;
@@ -30,13 +32,13 @@ sub handler {
 
     ok $port == $vars->{port};
 
-    my $serverroot = find_config_val($tree, 'ServerRoot');
+    my $documentroot = find_config_val($tree, 'DocumentRoot');
 
-    ok $serverroot;
+    ok $documentroot;
 
-    ok $serverroot eq qq("$vars->{serverroot}");
+    ok $documentroot eq qq("$vars->{documentroot}");
 
-    0;
+    Apache::OK;
 }
 
 sub find_config_val {
